<?php
/**
 * Testscript for Advanced Mod Rewrite Plugin.
 *
 * The goal of this testscript is to provide an easy way for a variance comparison
 * of created SEO URLs against their resolved parts.
 *
 * This testscript fetches the full category and article structure of actual
 * Contenido installation, creates the SEO URLs for each existing category/article
 * and resolves the generated URLs.
 *
 * Usage:
 * ------
 * 1. Install the Advanced Mod Rewrite Plugin
 * 2. Set your desired plugin settings by using the Advanced Mod Rewrite area
 *    in the Backend
 * 3. Copy this file to your client directory, it is normally named '/cms/'
 * 4. Exclude this file from rewriting, add following line in your .htaccess
 *    RewriteRule ^cms/mr_test.php.*$ - [L]
 * 5. Browse to http://host/cms/mr_test.php
 * 6. Disable the script after finished testing, see line 31
 *
 * @author      Murat Purc <murat@purc.de>
 * @copyright    Murat Purc 2008
 * @package     Contenido
 * @subpackage  ModRewrite
 */


// Uncomment following line, if you're finished with the tests
#die('Testscript disabled');


####################################################################################################
##### Contenido front_content.php header (compressed)

if (!defined("CON_FRAMEWORK")) { define("CON_FRAMEWORK", true); }
$contenido_path = '';
include_once ("config.php");
include_once ($contenido_path . 'classes/class.security.php');
Contenido_Security::checkRequests();
if (isset($_REQUEST['belang'])) {
    $aValid = array('de_DE', 'en_US', 'fr_FR', 'it_IT', 'nl_NL');
    if (!in_array(strval($_REQUEST['belang']), $aValid)) {
        die('Please use a valid language!');
    }
}
include_once ($contenido_path."includes/startup.php");
if ($cfg['http_params_check']['enabled'] === true) {
    cInclude('classes', 'class.httpinputvalidator.php');
    $oHttpInputValidator = new HttpInputValidator($cfg["path"]["contenido"] . $cfg["path"]["includes"] . '/config.http_check.php');
}
cInclude("includes", "functions.con.php");
cInclude("includes", "functions.con2.php");
cInclude("includes", "functions.api.php");
cInclude("includes", "functions.pathresolver.php");
if ($cfg["use_pseudocron"] == true){
    $oldpwd = getcwd();
    chdir($cfg["path"]["contenido"].$cfg["path"]["cronjobs"]);
    cInclude("includes", "pseudo-cron.inc.php");
    chdir($oldpwd);
}
if ($contenido) {
    page_open(array ('sess' => 'Contenido_Session', 'auth' => 'Contenido_Challenge_Crypt_Auth', 'perm' => 'Contenido_Perm'));
    i18nInit($cfg["path"]["contenido"].$cfg["path"]["locale"], $belang);
} else {
    page_open(array ('sess' => 'Contenido_Frontend_Session', 'auth' => 'Contenido_Frontend_Challenge_Crypt_Auth', 'perm' => 'Contenido_Perm'));
}
require_once $cfg['path']['contenido'] . $cfg['path']['includes'] . 'functions.includePluginConf.php';
#CEC_Hook::execute('Contenido.Frontend.AfterLoadPlugins');
$db = new DB_Contenido;
$sess->register("cfgClient");
$sess->register("errsite_idcat");
$sess->register("errsite_idart");
$sess->register("encoding");
if ($cfgClient["set"] != "set") {
    rereadClients();
}
$sql = "SELECT idlang, encoding FROM ".$cfg["tab"]["lang"];
$db->query($sql);
while ($db->next_record()) {
    $encoding[$db->f("idlang")] = $db->f("encoding");
}
if (is_numeric($tmpchangelang) && $tmpchangelang > 0) {
    $savedlang = $lang;
    $lang = $tmpchangelang;
}
if (isset($changeclient) && !is_numeric($changeclient)) {
    unset ($changeclient);
}
if (isset($client) && !is_numeric($client)) {
    unset ($client);
}
if (isset($changelang) && !is_numeric($changelang)) {
    unset ($changelang);
}
if (isset($lang) && !is_numeric($lang)) {
    unset ($lang);
}
if (isset($changeclient)){
    $client = $changeclient;
    unset($lang); unset($load_lang);
}
if (isset($changelang)) $lang = $changelang;
if (!isset($client)) {
    $client = $load_client;
}
if (!isset($lang)) {
    if (isset($load_lang)) {
        $lang = $load_lang;
    } else {
        $sql = "SELECT B.idlang FROM ".$cfg["tab"]["clients_lang"]." AS A, ".$cfg["tab"]["lang"]." AS B WHERE A.idclient='".Contenido_Security::toInteger($client)."' AND A.idlang = B.idlang LIMIT 0,1";
        $db->query($sql);
        $db->next_record();
        $lang = $db->f("idlang");
    }
}
if (!$sess->is_registered("lang") ) $sess->register("lang");
if (!$sess->is_registered("client") ) $sess->register("client");
if (isset ($username)) {
    $auth->login_if(true);
}
header("Content-Type: text/html; charset={$encoding[$lang]}");
if (isset ($logout)){
    $auth->logout(true);
    $auth->unauth(true);
    $auth->auth["uname"] = "nobody";
}
if (file_exists("config.local.php")) {
    @ include ("config.local.php");
}
if (isset($path) && strlen($path) > 1) {
    if ($cfg["urlpathresolve"] == true)     {
        $iLangCheck = 0;
        $idcat = prResolvePathViaURLNames($path, $iLangCheck);
    } else {
        $iLangCheck = 0;
        $idcat = prResolvePathViaCategoryNames($path, $iLangCheck);
        if($lang != $iLangCheck){
            $lang = $iLangCheck;
        }
    }
}
$aParams = array (
    'client' => $client, 'idcat' => $errsite_idcat[$client], 'idart' => $errsite_idart[$client],
    'lang' => $lang, 'error'=> '1'
);
$errsite = 'Location: ' . str_replace('&amp;', '&', Contenido_Url::getInstance()->build($aParams, true));
if ($idart && !$idcat && !$idcatart) {
    $sql = "SELECT idcat FROM ".$cfg["tab"]["cat_art"]." WHERE idart = '".Contenido_Security::toInteger($idart)."'";
    $db->query($sql);
    if ($db->next_record()) {
        $idcat = $db->f("idcat");
    }
}


####################################################################################################
##### Plugin related area


// set debug configuration, if you want to see the debug toolbar
$options = array(
    'enable'                    => true,
    'ressource_urls'            => array('/contenido/logs/errorlog.txt'),
    'dump_super_globals'        => array('$_GET', '$_POST', '$_COOKIE', '$_SESSION'),
    'ignore_empty_superglobals' => true,
    'magic_word'                => 'foobar'
);
$oDebug = Contenido_mpDebug::getInstance();
$oDebug->setConfig($options);


####################################################################################################
##### Form processing / Controller


$mrTestNoOptionSelected = false;
if (!_getRequest('idart') && !_getRequest('idcat') && !_getRequest('idcatart') && !_getRequest('idartlang')) {
    $mrTestNoOptionSelected = true;
}

$aData['form']['idart_chk']     = (_getRequest('idart')) ? ' checked="checked"' : '';
$aData['form']['idcat_chk']     = (_getRequest('idcat')) ? ' checked="checked"' : '';
$aData['form']['idcatart_chk']  = (_getRequest('idcatart')) ? ' checked="checked"' : '';
$aData['form']['idartlang_chk'] = (_getRequest('idartlang')) ? ' checked="checked"' : '';
$aData['form']['maxitems']      = _getRequest('maxitems', 200);


$oMrTestAction = new ModRewriteTestActions();

if ($mrTestNoOptionSelected) {
    $oMrTestAction->executeIndex();
} else {
    $oMrTestAction->executeTest();
}


if (isset($oDebug)) {
    if (is_array($GLOBALS['DB_Contenido_QueryCache']) && count($GLOBALS['DB_Contenido_QueryCache']) > 0) {
        $oDebug->addDebug($GLOBALS['DB_Contenido_QueryCache'], 'sql statements');
    }
    $aData['content'] .= $oDebug->getResults(false);
}


####################################################################################################
##### Contenido front_content.php footer

if (file_exists("config.after.php")) {@include("config.after.php");}
if (isset ($savedlang)){$lang = $savedlang;}
page_close();


####################################################################################################
#### Template / View

echo <<<HTML
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
   "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="de">
<head>
    <title>Test - Contenido Plugin Advanced Mod Rewrite</title>
    <meta http-equiv="Content-Type" content="application/xhtml+xml; charset=iso-8859-1" />
    <style type="text/css">
    body {margin:0; padding:0; background:#fff; color:#000; font-family:arial,helvetica,sans-serif; font-size:12px;}
    #wrap {margin:15px 10px;}
    h1 {font-size:16px;}
    form {width:70%; border:1px solid #999; background:#eaeaea; padding:0.5em; margin-bottom:15px;}
    fieldset {margin-bottom:10px;}
    fieldset p {margin:0;}
    form .chk {width:24%; float:left;}
    .left {float:left;}
    .right {float:right;}
    .clear {clear:both; font-size:0pt !important; height:0pt !important; line-height:0pt !important;}
    </style>
</head>
<body>
<div id="wrap">

<h1>Advanced Mod Rewrite test</h1>

<form name="mr_test" id="mr_test" action="{$_SERVER['PHP_SELF']}">
    <fieldset>
        <legend>Parameter to add</legend>
        <p></p>
        <div class="chk">
            <input type="checkbox" id="idart" name="idart" value="1"{$aData['form']['idart_chk']} /><label for="idart">idart</label>
        </div>
        <div class="chk">
            <input type="checkbox" id="idcat" name="idcat" value="1"{$aData['form']['idcat_chk']} /><label for="idcat">idcat</label>
        </div>
        <div class="chk">
            <input type="checkbox" id="idcatart" name="idcatart" value="1"{$aData['form']['idcatart_chk']} /><label for="idcatart">idcatart</label>
        </div>
        <div class="chk">
            <input type="checkbox" id="idartlang" name="idartlang" value="1"{$aData['form']['idartlang_chk']} /><label for="idartlang">idartlang</label>
        </div>
        <br class="clear" />
    </fieldset>
    <div style="margin:0.3em 0;">
        <div class="left">
            <label for="maxitems">Max items to process: </label><input type="text" id="maxitems" maxlength="4" name="maxitems" value="{$aData['form']['maxitems']}" />
        </div>
        <div class="right" style="padding-right:5px;">
            <input type="submit" name="test" value="run test" />
        </div>
        <br class="clear" />
    </div>
</form>

{$aData['content']}

</div>
</body>
</html>
HTML;


####################################################################################################
##### Classes / Functions


/**
 * Simple action controller.
 *
 * @author      Murat Purc <murat@purc.de>
 * @copyright    Murat Purc 2008
 * @package     Contenido
 * @subpackage  ModRewriteTest
 */
class ModRewriteTestActions {

    private $data;

    function __construct() {
        $this->data = & $GLOBALS['aData'];
    }

    /**
     * Execute index action
     */
    function executeIndex() {
        $this->data['content'] = '<pre>

Select several options from above and run the test.

<b>The result will be a outputlist like:</b>

Navigationame
builder in:    The URL beeing passed to the URL builder
builder out:   Result of rebuild URL, should be consistent with your settings
resolved url:  Resolved URL (the URL beeing returned by URL builder) by ModRewriteController
resolved data: Some resolved variables like $client, $lang, etc.

<b>Example:</b>

Navigationame
builder in:    front_content.php?idcat=1&foo=bar
builder out:   /deutsch-hauptnavigation.index.html?foo=bar
resolved url:  /deutsch/hauptnavigation/index.html?foo=bar
resolved data: client=1; lang=1; changelang=1; idcat=1
</pre>';

    }

    /**
     * Execute test action
     */
    function executeTest() {
        $this->data['content'] = '';

        // Array for testcases
        $aTests  = array();

        // Instance of mr test
        $oMRTest = new ModRewriteTest();

        $startTime = getmicrotime();

        // Fetch complete Contenido page structure
        $aStruct = $oMRTest->fetchFullStructure();
        Contenido_mpDebug::getInstance()->addDebug($aStruct, 'mr_test.php $aStruct');

        // Loop through the structure and compose testcases
        foreach ($aStruct as $idcat => $aCat) {
            // category
            $aTests[] = array(
                'url'   => $oMRTest->composeURL($aCat, 'c'),
                'level' => $aCat['level'],
                'name'  => $aCat['name']
            );

            foreach ($aCat['articles'] as $idart => $aArt) {
                // articles
                $aTests[] = array(
                    'url'  => $oMRTest->composeURL($aArt, 'a'),
                    'level' => $aCat['level'],
                    'name' => $aCat['name'] . ' :: ' . $aArt['title']
                );
            }
        }

        // compose content
        $this->data['content'] = '<pre>';

        $oMRUrlStack = ModRewriteUrlStack::getInstance();

        // first loop to add urls to mr url stack
        foreach ($aTests as $p => $v) {
            $oMRUrlStack->add($v['url']);
        }

        $successCounter = 0;
        $failCounter    = 0;
        
        // second loop to do the rest
        foreach ($aTests as $p => $v) {
            $url    = mr_buildNewUrl($v['url']);
            $arr    = $oMRTest->resolveUrl($url);
            $resUrl = $oMRTest->getResolvedUrl();
            $color  = 'green';

            if ($url !== $resUrl) {
                if ($oMRTest->getRoutingFoundState()) {
                    $successCounter++;
                    $resUrl = 'route to -&gt; ' . $resUrl;
                } else {
                    $color = 'red';
                    $failCounter++;
                }
            } else {
                $successCounter++;
            }
            
            $pref   = str_repeat('    ', $v['level']);

            $this->data['content'] .= "\n"
                . $pref . '<strong>' . $v['name'] . '</strong>' . "\n"
                . $pref . 'builder in:    ' . $v['url'] . "\n"
                . $pref . 'builder out:   <a href="' . $url . '" target="_blank">' . $url . '</a>'  . "\n"
                . $pref . '<span style="color:' . $color . '">resolved url:  ' . $resUrl . "</span>\n"
                . $pref . 'resolved data: ' . $oMRTest->getReadableResolvedData($arr) . "\n";
        }
        $this->data['content'] .= '</pre>';

        $totalTime = sprintf('%.4f', (getmicrotime() - $startTime));
        $msg = '<strong>Execution of test took: ' . $totalTime . ' seconds.<br />'
              . 'Number of processed URLs: ' . ($successCounter + $failCounter) . '<br />'
              . '<span style="color:green">Successfull resolved: ' . ($successCounter) . '</span><br />'
              . '<span style="color:red">Resolving failed: ' . ($failCounter) . '</span></strong><br /><br />';

        $this->data['content'] = $msg . $this->data['content'];

    }

}


/**
 * Advanced Mod Rewrite test class.
 *
 * @author      Murat Purc <murat@purc.de>
 * @copyright    Murat Purc 2008
 * @package     Contenido
 * @subpackage  ModRewriteTest
 */
class ModRewriteTest {

    /**
     * @var  array  Global $cfg array
     */
    private $cfg;

    /**
     * @var  array  Global $cfg['tab'] array
     */
    private $cfgTab;


    /**
     * @var  int  Max items to process
     */
    private $maxItems;

    /**
     * @var  string  Actual resolved url
     */
    private $sResolvedUrl;


    /**
     * @var  bool  Routing found flag
     */
    private $bRoutingFound = false;


    /**
     * Constuctor
     */
    function __construct() {
        $this->cfg    = & $GLOBALS['cfg'];
        $this->cfgTab = & $GLOBALS['cfg']['tab'];
        $this->maxItems = $GLOBALS['aData']['form']['maxitems'];
    }


    /**
     * Returns resolved URL
     *
     * @return  bool  Resolved URL
     */
    function getResolvedUrl(){
        return $this->sResolvedUrl;
    }


    /**
     * Returns flagz about found routing
     *
     * @return  bool
     */
    function getRoutingFoundState(){
        return $this->bRoutingFound;
    }


    /**
     * Fetchs full structure of the installation (categories and articles) and returns it back.
     *
     * @param   int    $idclient  Client id
     * @param   int    $idlang    Language id
     * @return  array  Full structure as follows
     * <code>
     *   $arr[idcat] = Category dataset
     *   $arr[idcat]['articles'][idart] = Article dataset
     * </code>
     */
    function fetchFullStructure($idclient=null, $idlang=null) {

        $db  = new DB_Contenido();
        $db2 = new DB_Contenido();

        if (!$idclient || (int) $idclient == 0) {
            $idclient = $GLOBALS['client'];
        }
        if (!$idlang || (int) $idlang == 0) {
            $idlang = $GLOBALS['lang'];
        }

        $aTab = $this->cfgTab;

        $aStruct = array();

        $sql = "SELECT
                    *
                FROM
                    " . $aTab['cat_tree'] . " AS a,
                    " . $aTab['cat_lang'] . " AS b,
                    " . $aTab['cat'] . " AS c
                WHERE
                    a.idcat = b.idcat AND
                    c.idcat = a.idcat AND
                    c.idclient = '".$idclient."' AND
                    b.idlang = '".$idlang."'
                ORDER BY
                    a.idtree";

        $db->query($sql);

        $loop    = false;
        $counter = 0;

        while ($db->next_record()) {

            if (++$counter == $this->maxItems) {
                break; // break this loop
            }

            $idcat = $db->f('idcat');
            $aStruct[$idcat] = $db->Record;
            $aStruct[$idcat]['articles'] = array();

            if ($this->cfg['is_start_compatible'] == true) {
                $compStatement = ' a.is_start DESC, ';
            } else {
                $compStatement = '';
            }

            $sql2 = "SELECT
                         *
                     FROM
                         ".$aTab['cat_art']."  AS a,
                         ".$aTab['art']."      AS b,
                         ".$aTab['art_lang']." AS c
                     WHERE
                         a.idcat = '".$idcat."' AND
                         b.idart = a.idart AND
                         c.idart = a.idart AND
                         c.idlang = '".$idlang."' AND
                         b.idclient = '".$idclient."'
                     ORDER BY
                         " . $compStatement . "
                         c.title ASC";

            $db2->query($sql2);

            while ($db2->next_record()) {
                $idart = $db2->f('idart');
                $aStruct[$idcat]['articles'][$idart] = $db2->Record;
                if (++$counter == $this->maxItems) {
                    break 2; // break this and also superior loop
                }
            }
        }

        return $aStruct;
    }


    /**
     * Creates an URL using passed data.
     *
     * The result is used to generate seo urls...
     *
     * @param  array  $arr  Assoziative array with some data as follows:
     * <code>
     * $arr['idcat']
     * $arr['idart']
     * $arr['idcatart']
     * $arr['idartlang']
     * </code>
     * @param  string  $type  Either 'c' or 'a' (category or article). If set to
     *                        'c' only the parameter idcat will be added to the URL
     */
    function composeURL($arr, $type) {
        $type = ($type == 'a') ? 'a' : 'c';

        $param = array();

        if ($type == 'c') {
            $param[] = 'idcat=' . $arr['idcat'];
        } else {
            if (_getRequest('idart')) {
                $param[] = 'idart=' . $arr['idart'];
            }
            if (_getRequest('idcat')) {
                $param[] = 'idcat=' . $arr['idcat'];
            }
            if (_getRequest('idcatart')) {
                $param[] = 'idcatart=' . $arr['idcatart'];
            }
            if (_getRequest('idartlang')) {
                $param[] = 'idartlang=' . $arr['idartlang'];
            }
        }
        $param[] = 'foo=bar';
        return 'front_content.php?' . implode('&amp;', $param);

    }


    /**
     * Resolves variables of an page (idcat, idart, idclient, idlang, etc.) by
     * processing passed url using ModRewriteController
     *
     * @param   string  $url  Url to resolve
     * @return  array   Assoziative array with resolved data
     */
    public function resolveUrl($url){

        // some globals to reset
        $aGlobs = array(
            'mr_preprocessedPageError', 'idart', 'idcat'
        );
        foreach ($aGlobs as $p => $k) {
            if (isset($GLOBALS[$k])) { unset($GLOBALS[$k]); }
        }

        $aReturn = array();

        // create an mod rewrite controller instance and execute processing
        $oMRController = new ModRewriteController($url);
        $oMRController->execute();

        if ($oMRController->errorOccured()) {

            // an error occured (idcat and or idart couldn't catched by controller)
            $aReturn['mr_preprocessedPageError'] = 1;

            $this->sResolvedUrl  = '';
            $this->bRoutingFound = false;

        } else {

            // set some global variables

            $this->sResolvedUrl  = $oMRController->getResolvedUrl();
            $this->bRoutingFound = $oMRController->getRoutingFoundState();

            if ($oMRController->getClient()) {
                $aReturn['client'] = $oMRController->getClient();
            }

            if ($oMRController->getChangeClient()) {
                $aReturn['changeclient'] = $oMRController->getChangeClient();
            }

            if ($oMRController->getLang()) {
                $aReturn['lang'] = $oMRController->getLang();
            }

            if ($oMRController->getChangeLang()) {
                $aReturn['changelang'] = $oMRController->getChangeLang();
            }

            if ($oMRController->getIdArt()) {
                $aReturn['idart'] = $oMRController->getIdArt();
            }

            if ($oMRController->getIdCat()) {
                $aReturn['idcat'] = $oMRController->getIdCat();
            }

            if ($oMRController->getPath()) {
                $aReturn['path'] = $oMRController->getPath();
            }

        }

        return $aReturn;
    }


    /**
     * Creates a readable string from passed resolved data array.
     *
     * @param   array   Assoziative array with resolved data
     * @return  string  Readable resolved data
     */
    public function getReadableResolvedData(array $data) {
        // compose resolved string
        $ret = '';
        foreach ($data as $k => $v) {
            $ret .= $k . '=' . $v . '; ';
        }
        $ret = substr($ret, 0, strlen($ret)-2);
        return $ret;
    }

}


/**
 * Minimalistic'n simple way to get request variables.
 *
 * Checks occurance in $_GET, then in $_POST. Uses trim() and strip_tags() to preclean data.
 *
 * @param   string  $key      Name of var to get
 * @param   mixed   $default  Default value to return
 * @return  mixed   The value
 */
function _getRequest($key, $default=null) {
    static $cache;
    if (!isset($cache)) {
        $cache = array();
    }
    if (isset($cache[$key])) {
        return $cache[$key];
    }
    if (isset($_GET[$key])) {
        $val = $_GET[$key];
    } elseif (isset($_POST[$key])) {
        $val = $_POST[$key];
    } else {
        $val = $default;
    }
    $cache[$key] = strip_tags(trim($val));
    return $cache[$key];
}
