<?php
/**
 * Includes base mod rewrite class.
 *
 * @author      Murat Purc <murat@purc.de>
 * @date        24.09.2008
 * @package     Contenido
 * @subpackage  ModRewrite
 */


defined('CON_FRAMEWORK') or die('Illegal call');


/**
 * Abstract base mod rewrite class.
 *
 * Provides some common features such as common debugging, globals/configuration
 * access for childs.
 *
 * @author      Murat Purc <murat@purc.de>
 * @date        24.09.2008
 * @package     Contenido
 * @subpackage  ModRewrite
 */
abstract class ModRewriteBase {

    /**
     * Debugger instance
     *
     * @var  Contenido_mpDebug
     */
    protected static $_oDebug;

    /**
     * Globals instance
     *
     * @var mpGlobals
     */
    protected static $_oGlobals;


    /**
     * Initialization, is to call at least once by an child.
     */
    protected static function initialize() {
        self::$_oDebug   = Contenido_mpDebug::getInstance();
        self::$_oGlobals = mpGlobals::getInstance();
    }


    /**
     * Returns enabled state of mod rewrite plugin
     *
     * @return  bool
     */
    public static function isEnabled(){
        return (self::getConfig('use', 0) == 1) ? true : false;
    }


    /**
     * Returns configuration of mod rewrite, content of gobal $cfg['mod_rewrite']
     *
     * @param   string  $key  Name of configuration key
     * @return  mixed   Desired value mr configuration, either the full configuration
     *                  or one of the desired subpart
     */
    public static function getConfig($key=null, $default=null) {
        $aCfg = self::$_oGlobals->get('cfg/mod_rewrite');
        if ($key == null) {
            return $aCfg;
        } elseif ((string) $key !== '') {
            return (isset($aCfg[$key])) ? $aCfg[$key] : $default;
        } else {
            return $default;
        }
    }

}
