<?php
/**
 * Plugin Advanced Mod Rewrite initialization file.
 *
 * This file will be included by Contenido plugin loader routine, and the content
 * of this file ensures that the AMR Plugin will be initialized correctly.
 *
 * @author      Murat Purc <murat@purc.de>
 * @copyright    Murat Purc 2008
 * @package     Contenido
 * @subpackage  ModRewrite
 */


defined('CON_FRAMEWORK') or die('Illegal call');


####################################################################################################
/**
 * Chain Contenido.Frontend.CreateURL
 * This chain is called inside some scripts (front_content.php) to create urls.
 *
 * @todo: Iis added to provide downwards compatibility for the amr plugin.
 *        There is no need for this chain since Contenido 4.8.9 contains its own Url building feature.
 * @deprecated
 *
 * Parameters & order:
 * string   URL including parameter value pairs
 *
 * Returns:
 * string 	Returns modified URL
 */
$_cecRegistry->registerChain("Contenido.Frontend.CreateURL", "string");
####################################################################################################


// initialize client id
if (isset($GLOBALS['client']) && (int) $GLOBALS['client'] > 0) {
    $clientId = (int) $GLOBALS['client'];
} elseif (isset($GLOBALS['load_client']) && (int) $GLOBALS['load_client'] > 0) {
    $clientId = (int) $GLOBALS['load_client'];
} else {
    $clientId = '';
}

// load language file (internationalization ist not implemented, just a pre release)
$pluginDir  = $GLOBALS['cfg']['path']['contenido'] . $GLOBALS['cfg']['path']['plugins'] . 'mod_rewrite/';
$pluginLang = ($GLOBALS['i18nLanguage'] == 'en_US') ? 'en_US' : 'de_DE';
if (is_file($pluginDir . 'locale/' . $pluginLang . '.inc.php')) {
    @include_once($pluginDir . 'locale/' . $pluginLang . '.inc.php');
} else {
    @include_once($pluginDir . 'locale/en_US.inc.php');
}


// include necessary sources
cInclude('classes', 'Debug/DebuggerFactory.class.php');
cInclude('classes', 'mp/class.mpdebug.php');
cInclude('classes', 'mp/class.mpglobals.php');
plugin_include('mod_rewrite', 'classes/class.modrewritebase.php');
plugin_include('mod_rewrite', 'classes/class.modrewrite.php');
plugin_include('mod_rewrite', 'classes/class.modrewritecontroller.php');
plugin_include('mod_rewrite', 'classes/class.modrewriteurlstack.php');
plugin_include('mod_rewrite', 'classes/class.modrewriteurlutil.php');
plugin_include('mod_rewrite', 'includes/functions.mod_rewrite.php');
cInclude('classes', 'mp/class.confighandler.php');


// set debug configuration
$options = array(
    'enable'                    => false, // disable debug
    'ressource_urls'            => array('/contenido/logs/errorlog.txt'),
    'dump_super_globals'        => array('$_GET', '$_POST', '$_COOKIE', '$_SESSION'),
    'ignore_empty_superglobals' => true
);

Contenido_mpDebug::getInstance()->setConfig($options);


// initialize mr plugin
ModRewrite::initialize($clientId);

if (ModRewrite::isEnabled()) {

    $aMrCfg = ModRewrite::getConfig();

    $_cecRegistry = cApiCECRegistry::getInstance();

    // Add new tree function to Contenido Extension Chainer
    $_cecRegistry->addChainFunction('Contenido.Action.str_newtree.AfterCall', 'mr_strNewTree');

    // Add move subtree function to Contenido Extension Chainer
    $_cecRegistry->addChainFunction('Contenido.Action.str_movesubtree.AfterCall', 'mr_strMoveSubtree');

    // Add new category function to Contenido Extension Chainer
    $_cecRegistry->addChainFunction('Contenido.Action.str_newcat.AfterCall', 'mr_strNewCategory');

    // Add rename category function to Contenido Extension Chainer
    $_cecRegistry->addChainFunction('Contenido.Action.str_renamecat.AfterCall', 'mr_strRenameCategory');

    // Add move up category function to Contenido Extension Chainer
    $_cecRegistry->addChainFunction('Contenido.Action.str_moveupcat.AfterCall', 'mr_strMoveUpCategory');

    // Add move down category function to Contenido Extension Chainer
    $_cecRegistry->addChainFunction('Contenido.Action.str_movedowncat.AfterCall', 'mr_strMovedownCategory');

    // Add copy category function to Contenido Extension Chainer
    $_cecRegistry->addChainFunction('Contenido.Category.strCopyCategory', 'mr_strCopyCategory');

    // Add category sync function to Contenido Extension Chainer
    $_cecRegistry->addChainFunction('Contenido.Category.strSyncCategory_Loop', 'mr_strSyncCategory');

    // Add save article (new and existing category) function to Contenido Extension Chainer
    $_cecRegistry->addChainFunction('Contenido.Action.con_saveart.AfterCall', 'mr_conSaveArticle');

    // Add move article function to Contenido Extension Chainer
    $_cecRegistry->addChainFunction('Contenido.Article.conMoveArticles_Loop', 'mr_conMoveArticles');

    // Add duplicate article function to Contenido Extension Chainer
    $_cecRegistry->addChainFunction('Contenido.Article.conCopyArtLang_AfterInsert', 'mr_conCopyArtLang');

    // Add sync article function to Contenido Extension Chainer
    $_cecRegistry->addChainFunction('Contenido.Article.conSyncArticle_AfterInsert', 'mr_conSyncArticle');

    if (!isset($GLOBALS['contenido'])) {
        // we are not in backend, add cec functions for rewriting

        // Add mr related function for hook "after plugins loaded" to Contenido Extension Chainer
        $_cecRegistry->addChainFunction('Contenido.Frontend.AfterLoadPlugins', 'mr_runFrontendController');

        // Add url rewriting function to Contenido Extension Chainer
        // @todo: no more need since Contenido 4.8.9 provides central Url building,
        //        but it is still available  because of downwards compatibility
        // @deprecated
        $_cecRegistry->addChainFunction('Contenido.Frontend.CreateURL', 'mr_buildNewUrl');

        // overwrite url builder configuration with own url builder
        $GLOBALS['cfg']['url_builder']['name'] = 'MR';
        $GLOBALS['cfg']['config']              = array();
        cInclude('classes', 'Url/Contenido_Url.class.php');
        cInclude('classes', 'UrlBuilder/Contenido_UrlBuilderConfig.class.php');
        Contenido_UrlBuilderConfig::setConfig($cfg['url_builder']);

        if ($aMrCfg['rewrite_urls_at_congeneratecode'] == 1) {

            // Add url rewriting at code generation to Contenido Extension Chainer
            $_cecRegistry->addChainFunction('Contenido.Content.conGenerateCode', 'mr_buildGeneratedCode');

        } elseif ($aMrCfg['rewrite_urls_at_front_content_output'] == 1) {

            // Add url rewriting at html output to Contenido Extension Chainer
            $_cecRegistry->addChainFunction('Contenido.Frontend.HTMLCodeOutput', 'mr_buildGeneratedCode');

        } else {

            // Fallback solution: Add url rewriting at code generation to Contenido Extension Chainer
            $_cecRegistry->addChainFunction('Contenido.Content.conGenerateCode', 'mr_buildGeneratedCode');

        }

    }

}

unset($clientId, $options);
